/******************************************************************************
*******************************************************************************
**                                                                           **
**      MOST for C Swing and Gopher - A file viewer                          **
**                                                                           **
*******************************************************************************
**                                                                           **
**      Original C coding by: John E. Davis                                  **
**                                                                           **
**      Modifications primarily by Foteos Macrides                           **
**                                                                           **
**      Copyright 1991, John E. Davis                                        **
**                                                                           **
**      Terms of Copyright: See MAIN.C                                       **
**                                                                           **
**      Disclaimer: No warranties as to the use or reliability of this       **
**          software are given by the authors or distributors.  Use at       **
**          your own risk.                                                   **
**                                                                           **
*******************************************************************************
******************************************************************************/
/*
 *  MOST:  cmd.c
 *
 *  Modified for C SWING and GOPHER compatibility by Foteos Macrides
 *                                                MACRIDES@SCI.WFEB.EDU
 */

#include <stdio.h>
#include <string.h>
#include <processes.h>
#include "sysdep.h"
#include "search.h"
#include "window.h"
#include "file.h"
#include "keym.h"
#include "externs.h"

/*
**  extended command key (':') processing
**  returns zero if not quit
*/
int do_extended_key()
{
    char ch, och, junk[136];
    int quit = 0, n, m_bell = 0;
    unsigned char *save;
    
    select_minibuffer();
    putc(':',stdout);
    fflush(stdout);
    ch = getkey();
    putc(ch,stdout);
    fflush(stdout);

    switch (ch)
      {
      /*
      **  fetch Next file in ring
      */
      case 'n':
      case 'N':
          if (!NUM_FILES)
            {
                most_message("File ring is empty.",1);
                quit = 1;
            }
          else if (NUM_FILES == 1)
            {
                most_message("File ring has only one file.",1);
                quit = 1;
            }
          else
            {
                quit = next_file();
                if (quit == -1)
                  {
                      quit = 0;
                      most_message("File fetch cancelled",0);
                  }
            }
          break;

      /*
      **  toggle Case-sensitivity of searches
      */
      case 'c':
      case 'C':
          MOST_C_OPT = !MOST_C_OPT;
          if (MOST_C_OPT)
            most_message("Searches now respect case.",0);
          else
            most_message("Searches no longer respect case.",0);
          break;

      /*
      **  Delete (from disk) the current file
      **  (+d extended switch must have been set)
      */
      case 'd':
      case 'D':
          if (MOST_D_OPT) /* +d argument is set, do try to delete */
            {
                /* notice the missing D. */
                fprintf(stdout,"elete %s? [n]:", BUF->file);
                fflush(stdout);
                ch = getkey();
                if (ch == 'y' || ch == 'Y')
                  {
                      if (!sys_delete_file(BUF->file))
                        most_message("File could not be deleted.",1);
                      else
                        most_message("File deleted.",0);
                  }
                else
                  most_message("File not deleted.",1);
            }
          else
            {
                most_message("File deletion (+d) is not enabled.",1);
            }
          break;

      /*
      **  toggle other Options
      */
      case 'o':
      case 'O':
          fputs("\rToggle option (b i s t v w): ",stdout); fflush(stdout);
          och = getkey();
          switch (och)
            {
            /*
            **  toggle forced Binary display mode
            */
            case 'b':
            case 'B':
                MOST_B_OPT = !MOST_B_OPT;
                NUM_LINES = count_lines(BEG,EOB);
                if (MOST_B_OPT)
                  most_message("Forced Binary Display mode on.",0);
                else
                  most_message("Forced Binary Display mode off.",0);
                break;

            /*
            **  if digit(s) preceded the ':' set selective
            **    Indentation display to skip lines indented
            **    beyond that column
            **  else turn off selective Indentation display
            */
            case 'i':
            case 'I':
                if (DIGIT_ARG == NULL)
                  n = 0;
                else
                  n = abs( *DIGIT_ARG );
                if (SELECTIVE_I_DISPLAY != n)
                  {
                      SELECTIVE_I_DISPLAY = n;
                      NUM_LINES = count_lines(BEG,EOB);
                  }
                if (SELECTIVE_I_DISPLAY)
                  {
                        sprintf(junk,
                          "Selective Indentation Display set at column %u.",n);
                        if (MOST_W_OPT)
                          {
                              strcat(junk,"  Disable Wrap to implement.");
                              m_bell = 1;
                          }
                  }
                else
                  {
                      strcpy(junk,"Selective Indentation Display off.");
                      strcat(junk,"  Precede the :O with a number to set.");
                  }
                most_message(junk,m_bell);
                break;

            /*
            **  toggle Squeezing of multiple blank lines to one
            */
            case 's':
            case 'S':
                MOST_S_OPT = !MOST_S_OPT;
                NUM_LINES = count_lines(BEG,EOB);
                if (MOST_S_OPT)
                  most_message("Squeeze Multiple Blank Lines on.",0);
                else
                  most_message("Squeeze Multiple Blank Lines off.",0);
                break;

            /*
            **  toggle display of Tabs as ^I
            **  (Verbose mode must also be set to implement)
            */
            case 't':
            case 'T':
                MOST_T_OPT = !MOST_T_OPT;
                if (!MOST_T_OPT)
                  most_message("Display Tabs as ^I off.",0);
                else
                  {
                      strcpy(junk,"Display Tabs as ^I on.");
                      if (!MOST_V_OPT)
                        {
                            strcat(junk,"  Toggle Verbose mode to implement.");
                            m_bell = 1;
                        }
                      most_message(junk,m_bell);
                  }
                break;

            /*
            **  toggle display of control characters as ^ch
            **  (Verbose mode)
            */
            case 'v':
            case 'V':
                MOST_V_OPT = !MOST_V_OPT;
                if (MOST_V_OPT)
                  most_message("Verbose (display control chars) mode on.",0);
                else
                  most_message("Verbose (display control chars) mode off.",0);
                break;

            /*
            **  toggle Wrapping mode
            */
            case 'w':
            case 'W':
                MOST_W_OPT = !MOST_W_OPT;
                NUM_LINES = count_lines(BEG,EOB);
                if (MOST_W_OPT)
                  {
                      strcpy(junk,"Wrapping on.");
                      if (SELECTIVE_I_DISPLAY)
                        {
                            strcat(junk,
                                  "  Selective Identation Display suppressed.");
                            m_bell = 1;
                        }
                      most_message(junk,m_bell);
                  }
                else
                  {
                      strcpy(junk,"Wrapping off.");
                      if (SELECTIVE_I_DISPLAY)
                        {
                            strcat(junk,
                                 "  Selective Identation Display implemented.");
                            m_bell = 1;
                        }
                      most_message(junk,m_bell);
                  }
                break;

            default:
                most_message("Options not modified.",1);
                break;
            }
          /*
          **  break point for switch (och), clean up and return
          */
          if (!NUM_LINES) NUM_LINES = 1;
          save_win_flags(WIN);
          save = C_POS;
          C_POS = BEG;
          C_LINE = 1;
          C_LINE = what_line(save);
          C_POS = save;
          WIN->beg_line = C_LINE;
          delete_line(1);
          exit_minibuffer();
          redraw_window();
          update_status(0);
          return(quit);
          break;

      default:
          putc('\007',stdout);
          delete_line(1);
          break;
      }
    /*
    **  break point for switch (ch), clean up and return
    */
    fflush(stdout);
    exit_minibuffer();
    return(quit);
}

/*
** prompted ('Cmd: ') extended command processing
** returns zero if not quit
*/
int do_extended_cmd()
{
    char most_cmd[136];
    int status, quit = 0;
    globalref int captive;

    most_cmd[0] = '\0';
    /*
    ** prompt for a command
    ** return if abort (^G) issued
    */
    if ((read_from_minibuffer("Cmd: ", most_cmd)) == -1)
      {
          most_message("Extended (system) command cancelled.", 0);
          return(0);
      }
          
    /*
    ** return if no command was entered
    */
    if (!strlen(most_cmd))
      {
          most_message("Extended (system) command cancelled.", 1);
          return(0);
      }
          
    /*
    ** Change Default directory
    */
    if (!strncasecmp(most_cmd, "cd" , 2))
      {
          if (SecureMode)
	    most_message(
	    "Sorry, you cannot change the default directory in secure mode.",1);
	  else
            most_cd();
      }
      
    /*
    ** Quit (Exit) from most
    */
    else if ((!strncasecmp(most_cmd,"quit", strlen(most_cmd))) || 
	     (!strncasecmp(most_cmd,"exit", strlen(most_cmd))))
      {
          quit = -1;
      }
      
    /*
    ** assume a system command and
    **   issue it via a spawn
    */
    else
      {
          if (SecureMode || captive)
	       most_message("Sorry, this function is disabled.",1);
          else
            {
               if (reset_most_tty())
                 {
                    reset_display();
                    fflush(stdout);
                    cls();
               
                    if (!do_most_system_command(most_cmd, 1))
                         quit = -1;

                    if (init_most_tty())
                         redraw_display();
                    else
                      {
                         quit = -1;
                         fputs("\nError re-initializing the terminal!!!\n",
                               stderr);
                      }
                 }
            }
      }

    return(quit);
}
